import sys
import os
from pathlib import Path
from .config import navigatorConfig

def is_virtualenv():
    return (hasattr(sys, 'real_prefix') or (hasattr(sys, 'base_prefix') and sys.base_prefix != sys.prefix))

# PROJECT PATH IS DEFINED?
SITE_ROOT = os.getenv('SITE_ROOT', None)

if not SITE_ROOT:
    # get Project PATH
    if is_virtualenv():
        SITE_ROOT = Path(sys.prefix).resolve().parent
    else:
        SITE_ROOT = Path(os.path.abspath(os.path.dirname(__file__))).resolve().parent.parent
    if not SITE_ROOT:
        SITE_ROOT = Path(sys.prefix).resolve().parent
else:
    SITE_ROOT = Path(SITE_ROOT).resolve()

# adding BASE_DIR for compatibility with Django
BASE_DIR = os.getenv('BASE_DIR', None)
if not BASE_DIR:
    BASE_DIR = SITE_ROOT
else:
    BASE_DIR = Path(BASE_DIR).resolve()

# for running DataIntegrator
SERVICES_DIR = BASE_DIR.joinpath('services')
SETTINGS_DIR = BASE_DIR.joinpath('settings')
EXTENSION_DIR = BASE_DIR.joinpath('extensions')

config = navigatorConfig(SITE_ROOT)

ENV = config.ENV
DEBUG = os.getenv('DEBUG', False)
# SECURITY WARNING: keep the secret key used in production secret!
PRODUCTION = config.getboolean('PRODUCTION', fallback=bool(not DEBUG))
LOCAL_DEVELOPMENT = (DEBUG == True and sys.argv[0] == 'run.py')

# Add Path Navigator to Sys path
sys.path.append(str(BASE_DIR))
sys.path.append(str(SERVICES_DIR))
sys.path.append(str(SETTINGS_DIR))
sys.path.append(str(EXTENSION_DIR))

from .conf import *



import os
import importlib
import sys
import types
import logging
from configparser import RawConfigParser, ConfigParser
from pathlib import Path
from dotenv import load_dotenv, dotenv_values
from asyncdb.providers.mcache import mcache
from asyncdb.providers.mredis import mredis


#### TODO: Feature Toggles
class Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]

    def __new__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__new__(cls, *args, **kwargs)
            setattr(cls, '__initialized', True)
        return cls._instances[cls]

class navigatorConfig(metaclass=Singleton):
    """
    navigatorConfig.

        Class for Navigator configuration
    """
    _self = None
    _ini = None
    _mem = None
    _redis = None
    ENV = ''
    _path = '/etc/troc/'
    _conffile = 'navigator.ini'
    _site_path = ''
    _debug = False
    __initialized = False

    def __del__(self):
        if self._mem:
            self._mem.close()

    def __init__(self, site_root=None):
        if(self.__initialized): return
        self.__initialized = True
        # this only load at first time
        if not site_root:
            site_root = Path(__file__).resolve().parent.parent
        self._site_path = site_root
        # get the current environment
        environment = os.getenv('ENV', '')
        self.ENV = environment
        # getting type of enviroment consummer:
        env_type = os.getenv('NAVCONFIG_ENV', 'file') # file by default
        # get the environment
        self.load_enviroment(env_type)
        # and get the config file declared in the environment file
        config_file = os.getenv('CONFIG_FILE', '/etc/troc/navigator.ini')
        self._ini = ConfigParser()
        #self._ini = RawConfigParser()
        cf = Path(config_file).resolve()
        if not cf.exists():
            cf = site_root.joinpath('etc', self._conffile)
        try:
            #with open(cf) as f:
            self._ini.read(cf)
        except IOError:
            print(cf, "INI file does not exist!")
            raise IOError("INI file does not exist!")
            return None
        # define debug
        self._debug = os.getenv('DEBUG', False)
        # get redis connection
        try:
            redis = {
                "host": os.getenv('CACHEHOST', 'localhost'),
                "port": os.getenv('CACHEPORT', 6379),
                "db": os.getenv('QUERYSET_DB', 0),
            }
            self._redis = mredis(params=redis)
            self._redis.connection()
            if self._debug:
                print("Redis Connected: {}".format(self._redis.is_connected()))
        except Exception as err:
            print(err)
        # get memcache SERVER
        try:
            mem_params = {
                "host": os.getenv('MEMCACHE_HOST', 'localhost'),
                "port": os.getenv('MEMCACHE_PORT', 11211)
            }
            self._mem = mcache(params=mem_params)
            self._mem.connection()
            if self._debug:
                print("Memcache Connected: {}".format(self._mem.is_connected()))
        except Exception as err:
            print(err)
            # memcache not working
            self._mem = None

    def save_environment(self, env_type: str = 'drive'):
        env_path = self.site_root.joinpath('env', self.ENV, '.env')
        # pluggable types
        if env_type == 'drive':
            from navconfig.loaders import driveLoader
            try:
                d = driveLoader()
                d.save_enviroment(env_path)
            except Exception as err:
                print('Error Reading Environment from Google Drive', err)

    def load_enviroment(self, env_type: str = 'file'):
        if env_type == 'file':
            env_path = self.site_root.joinpath('env', self.ENV, '.env')
            # warning if env_path is an empty file or doesnt exists
            if env_path.exists():
                if os.stat(str(env_path)).st_size == 0:
                    raise FileExistsError('Empty Environment File: {}'.format(env_path))
                # load dotenv
                load_dotenv(dotenv_path=env_path)
            else:
                raise FileNotFoundError('Environment file not found: {}'.format(env_path))
        else:
            # pluggable types
            if env_type == 'drive':
                from navconfig.loaders import driveLoader
                try:
                    d = driveLoader()
                    d.load_enviroment()
                except Exception as err:
                    print('Error Reading from Google Drive', err)

    @property
    def site_root(self):
        return self._site_path

    @property
    def ini(self):
        """
        ini.
            Returns a INI parser instance
        """
        return self._ini

    def addFiles(self, files):
        """
        addFiles.
            Add new files to the ini parser
        """
        self._ini.read(files)

    def addEnv(self, file):
        if file.exists() and file.is_file():
            load_dotenv(dotenv_path=file)

    def getboolean(self, value, section=None, fallback=None):
        """
        getboolean.
            Interface for getboolean function of ini parser
        """
        val = None
        # get ENV value
        if value in os.environ:
            val = os.getenv(value, fallback)
            if val:
                if val.lower() in self._ini.BOOLEAN_STATES: # Check inf val is Boolean
                    return self._ini.BOOLEAN_STATES[val.lower()]
                else:
                    return bool(val)

        # if not val and if section, get from INI
        if not val and section != None:
            try:
                val = self._ini.getboolean(section, value)
                if not val:
                    return fallback
                else:
                    return bool(val)
            except Exception:
                return fallback

        # If not val and not section, get from MEMCACHED
        if not val and section == None:
             #TODO: change to a non-async MEMCACHED connector
            val = self._mem.get(value)

        # last: check if value exists on ini
        for section in self._ini.sections():
            try:
                val = self._ini.get(section, value)
                if val:
                    if val.lower() in self._ini.BOOLEAN_STATES: # Check inf val is Boolean
                        return self._ini.BOOLEAN_STATES[val.lower()]
            except Exception:
                continue
        return fallback

    def getint(self, value, section=None, fallback=None):
        """
        getint.
            Interface for getint function of ini parser
        """
        val = None
        # get ENV value
        if value in os.environ:
            val = os.getenv(value, fallback)
            if val:
                return int(val)

        # if not val and if section, get from INI
        if not val and section is not None:
            try:
                val = self._ini.getint(section, value)
                if not val:
                    return fallback
                else:
                    return int(val)
            except Exception:
                return fallback

        # If not val and not section, get from MEMCACHED
        if not val and section is None:
            # TODO: change to a non-async MEMCACHED connector
            val = self._mem.get(value)
            if val:
                return int(val)

        # last: check if value exists on ini
        for section in self._ini.sections():
            try:
                val = self._ini.get(section, value)
                if val:
                    if val.isdigit(): # Check if val is Integer
                        return int(val)
            except Exception:
                continue
        return fallback

    def get(self, key, section=None, fallback=None):
        """
        get.
            Interface for get variable from differents sources
        """
        val = None
        # get ENV value
        if key in os.environ:
            val = os.getenv(key, fallback)
            if val:
                return val
        # if not in os.environ, got from Redis
        if not val and section is None:
            if self._redis.exists(key):
                return self._redis.get(key)
        # If not in redis, get from MEMCACHED
        if not val and section is None:
            val = self._mem.get(key)
            if val:
                return val
        # if not val and if section, get from INI
        if not val and section is not None:
            try:
                val = self._ini.get(section, key)
                if not val:
                    return fallback
                else:
                    return val
            except Exception:
                return fallback
        # last: check if value exists on ini
        for section in self._ini.sections():
            try:
                val = self._ini.get(section, key)
                if val:
                    return val
            except Exception:
                continue
        return fallback

    """
    Config Magic Methods (dict like)
    """

    def __getitem__(self, key):
        """
        Sequence-like operators
        """
        if key in os.environ:
            return os.getenv(key)
        elif self._redis.exists(key):
            return self._redis.get(key)
            # check if exists on memcached
        else:
            val = self._mem.get(key)
            if val:
                return val
            else:
                return None

    def __contains__(self, key):
        if key in os.environ:
            return True
        if self._redis.exists(key):
            return True
        val = self._mem.get(key)
        if val:
            return True
        else:
            return False

    ## attribute name
    def __getattr__(self, key):
        if key in os.environ:
            val = os.getenv(key)
        elif self._redis.exists(key):
            val = self._redis.get(key)
        else:
            val = self._mem.get(key)
        if val:
            try:
                if val.lower() in self._ini.BOOLEAN_STATES:
                    return self._ini.BOOLEAN_STATES[val.lower()]
                elif val.isdigit():
                    return int(val)
            finally:
                return val
        else:
            # if hasattr(self, key):
            #     return super(navigatorConfig, self).__getattr__(key)
            raise TypeError("NavigatorConfig Error: has not attribute {}".format(key))
        return None

    def set(self, key, value):
        """
        set
            set an enviroment variable on redis
        """
        return self._redis.set(key, value)



import os
import sys
import logging
from pathlib import Path

from .config import navigatorConfig
config = navigatorConfig()

DEBUG = os.getenv('DEBUG', False)
BASE_DIR = Path(os.getenv('BASE_DIR', config.site_root)).resolve()
SETTINGS_DIR = BASE_DIR.joinpath('settings')

if str(BASE_DIR) not in sys.path:
    sys.path.append(str(BASE_DIR))

if str(SETTINGS_DIR) not in sys.path:
    sys.path.append(str(SETTINGS_DIR))


try:
    from settings.settings import *
except (ImportError, ModuleNotFoundError):
    # running inside django project:
    print('Its recommended to use a settings/settings module to customize Navigator Configuration')

"""
User Local Settings
"""
try:
    from settings.local_settings import *
except (ImportError, ModuleNotFoundError):
    pass

"""
Logging
"""
logdir = config.get('logdir', section='logging', fallback='/tmp')
if DEBUG:
    loglevel = logging.DEBUG
else:
    loglevel = logging.INFO
logging_config = dict(
    version = 1,
    formatters = {
        "console": {
            'format': '%(message)s'
        },
        "file": {
            "format": "%(asctime)s: [%(levelname)s]: %(pathname)s: %(lineno)d: \n%(message)s\n"
        },
        'default': {
            'format': '[%(levelname)s] %(asctime)s %(name)s: %(message)s'}
        },
    handlers = {
        "console": {
                "formatter": "console",
                "class": "logging.StreamHandler",
                "stream": "ext://sys.stdout",
                'level': loglevel
        },
        'StreamHandler': {
                'class': 'logging.StreamHandler',
                'formatter': 'default',
                'level': loglevel
        },
        'RotatingFileHandler': {
                'class': 'logging.handlers.RotatingFileHandler',
                'filename': '{0}/{1}.log'.format(logdir, 'query_api'),
                'maxBytes': (1048576*5),
                'backupCount': 2,
                'encoding': 'utf-8',
                'formatter': 'file',
                'level': loglevel}
        },
    root = {
        'handlers': ['StreamHandler','RotatingFileHandler'],
        'level': loglevel,
        },
)



import os
import logging
from navconfig.loaders.base import BaseLoader
from pydrive.auth import GoogleAuth
from pydrive.drive import GoogleDrive
logging.getLogger('googleapiclient.discovery_cache').setLevel(logging.CRITICAL)


class driveLoader(BaseLoader):
    drive = None
    file_id = None

    def __init__(self, **kwargs):
        self.file_id = os.getenv('NAVCONFIG_DRIVE_ID')
        client = os.getenv('NAVCONFIG_DRIVE_CLIENT')
        if self.file_id and client:
            gauth = GoogleAuth()
            gauth.LoadCredentialsFile(client)
            if gauth.credentials is None:
                # Authenticate if they're not there
                gauth.LocalWebserverAuth() # Creates local webserver and auto handles authentication.
                # Save the current credentials to a file
            elif gauth.access_token_expired:
                gauth.Refresh()
            else:
                gauth.Authorize()
            gauth.SaveCredentialsFile(client)
            print("Google Auth Success")
            self.drive = GoogleDrive(gauth)
        else:
            raise Exception('Config Google Drive Error: you need to provide **NAVCONFIG_DRIVE_CLIENT** for client configuration')

    def load_enviroment(self):
        try:
            env = self.drive.CreateFile({'id': self.file_id})
            content = env.GetContentString()
            if content:
                self.load_from_string(content)
        except Exception as err:
            raise Exception('Error loading Environment: {}'.format(err))

    def save_enviroment(self, path:str=None):
        try:
            env = self.drive.CreateFile({'id': self.file_id})
            content = env.GetContentString()
            if content:
                with open(path, 'w+') as f:
                    f.write(content)
        except Exception as err:
            raise Exception('Error Saving Environment: {}'.format(err))



#!/usr/bin/env python
"""NavConfig.

    Configuration Service for Navigator and DataIntegrator.
See:
https://github.com/phenobarbital/NavConfig
"""

from setuptools import setup, find_packages

setup(
    name='navconfig',
    version=open("VERSION").read().strip(),
    python_requires=">=3.7.0",
    url='https://github.com/phenobarbital/NavConfig',
    description='Configuration tool for Navigator Services',
    long_description='Configuration tool for Navigator Services',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'Programming Language :: Python :: 3.7',
    ],
    author='Jesus Lara',
    author_email='jlara@trocglobal.com',
    packages=find_packages(),
    install_requires=[
        'asyncio==3.4.3',
        'python-dotenv==0.15.0',
        'configparser==5.0.1',
        'PyDrive==1.3.1',
        'pylibmc==1.6.1',
        'objectpath==0.6.1',
        'asyncdb'
    ],
    dependency_links=[
        'git+https://github.com/phenobarbital/asyncdb.git@master#egg=asyncdb'
    ],
    project_urls={  # Optional
        'Source': 'https://github.com/phenobarbital/NavConfig',
        'Funding': 'https://paypal.me/phenobarbital',
        'Say Thanks!': 'https://saythanks.io/to/phenobarbital',
    },
)
